@@ -6,7 +6,9 @@
 """Tests the handling of the Atom content when doing a POST Col-IRI."""
 
 from io import BytesIO
+import textwrap
 import uuid
+import warnings
 
 import attr
 from django.urls import reverse_lazy as reverse
@@ -22,18 +24,86 @@ from swh.deposit.config import (
 from swh.deposit.models import Deposit, DepositCollection, DepositRequest
 from swh.deposit.parsers import parse_xml
 from swh.deposit.tests.common import post_atom
-from swh.deposit.utils import compute_metadata_context
-from swh.model.identifiers import SWHID, parse_swhid
+from swh.deposit.utils import compute_metadata_context, extended_swhid_from_qualified
+from swh.model.hypothesis_strategies import (
+    directories,
+    present_contents,
+    releases,
+    revisions,
+    snapshots,
+)
+from swh.model.identifiers import ObjectType, QualifiedSWHID
 from swh.model.model import (
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
-    MetadataTargetType,
+    Origin,
     RawExtrinsicMetadata,
 )
 from swh.storage.interface import PagedResult
 
 
+def _insert_object(swh_storage, swhid):
+    """Insert an object with the given swhid in the archive"""
+    if swhid.object_type == ObjectType.CONTENT:
+        with warnings.catch_warnings():
+            # hypothesis doesn't like us using .example(), but we know what we're doing
+            warnings.simplefilter("ignore")
+            obj = present_contents().example()
+        swh_storage.content_add([attr.evolve(obj, sha1_git=swhid.object_id)])
+    else:
+        object_type_name = swhid.object_type.name.lower()
+        strategy = {
+            "directory": directories,
+            "revision": revisions,
+            "release": releases,
+            "snapshot": snapshots,
+        }[object_type_name]
+        method = getattr(swh_storage, object_type_name + "_add")
+        with warnings.catch_warnings():
+            # hypothesis doesn't like us using .example(), but we know what we're doing
+            warnings.simplefilter("ignore")
+            obj = strategy().example()
+        method([attr.evolve(obj, id=swhid.object_id)])
+
+
+def _assert_deposit_info_on_metadata(
+    swh_storage, metadata_swhid, deposit, metadata_fetcher
+):
+    swh_authority = MetadataAuthority(
+        MetadataAuthorityType.REGISTRY,
+        "http://deposit.softwareheritage.example/",
+        metadata=None,
+    )
+    page_results = swh_storage.raw_extrinsic_metadata_get(metadata_swhid, swh_authority)
+
+    assert len(page_results.results) == 1
+    assert page_results.next_page_token is None
+
+    expected_xml_data = textwrap.dedent(
+        f"""\
+        <deposit xmlns="https://www.softwareheritage.org/schema/2018/deposit">
+            <deposit_id>{deposit.id}</deposit_id>
+            <deposit_client>https://hal-test.archives-ouvertes.fr/</deposit_client>
+            <deposit_collection>test</deposit_collection>
+        </deposit>
+        """
+    )
+    assert page_results == PagedResult(
+        results=[
+            RawExtrinsicMetadata(
+                target=metadata_swhid,
+                discovery_date=deposit.complete_date,
+                authority=swh_authority,
+                fetcher=attr.evolve(metadata_fetcher, metadata=None),
+                format="xml-deposit-info",
+                metadata=expected_xml_data.encode(),
+            )
+        ],
+        next_page_token=None,
+    )
+
+
 def test_post_deposit_atom_201_even_with_decimal(
     authenticated_client, deposit_collection, atom_dataset
 ):
@@ -452,81 +522,47 @@ def test_deposit_metadata_fails_functional_checks(
 
 
 @pytest.mark.parametrize(
-    "swhid,target_type",
+    "swhid",
     [
-        (
-            "swh:1:cnt:01b5c8cc985d190b5a7ef4878128ebfdc2358f49",
-            MetadataTargetType.CONTENT,
-        ),
-        (
-            "swh:1:dir:11b5c8cc985d190b5a7ef4878128ebfdc2358f49",
-            MetadataTargetType.DIRECTORY,
-        ),
-        (
-            "swh:1:rev:21b5c8cc985d190b5a7ef4878128ebfdc2358f49",
-            MetadataTargetType.REVISION,
-        ),
-        (
-            "swh:1:rel:31b5c8cc985d190b5a7ef4878128ebfdc2358f49",
-            MetadataTargetType.RELEASE,
-        ),
-        (
-            "swh:1:snp:41b5c8cc985d190b5a7ef4878128ebfdc2358f49",
-            MetadataTargetType.SNAPSHOT,
-        ),
-        (
-            "swh:1:cnt:51b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
-            MetadataTargetType.CONTENT,
-        ),
-        (
-            "swh:1:dir:c4993c872593e960dc84e4430dbbfbc34fd706d0;origin=https://inria.halpreprod.archives-ouvertes.fr/hal-01243573;visit=swh:1:snp:0175049fc45055a3824a1675ac06e3711619a55a;anchor=swh:1:rev:b5f505b005435fa5c4fa4c279792bd7b17167c04;path=/",  # noqa
-            MetadataTargetType.DIRECTORY,
-        ),
-        (
-            "swh:1:rev:71b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
-            MetadataTargetType.REVISION,
-        ),
-        (
-            "swh:1:rel:81b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
-            MetadataTargetType.RELEASE,
-        ),
-        (
-            "swh:1:snp:91b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
-            MetadataTargetType.SNAPSHOT,
-        ),
+        "swh:1:cnt:01b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:dir:11b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:rev:21b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:rel:31b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:snp:41b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:cnt:51b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:dir:c4993c872593e960dc84e4430dbbfbc34fd706d0;origin=https://inria.halpreprod.archives-ouvertes.fr/hal-01243573;visit=swh:1:snp:0175049fc45055a3824a1675ac06e3711619a55a;anchor=swh:1:rev:b5f505b005435fa5c4fa4c279792bd7b17167c04;path=/",  # noqa
+        "swh:1:rev:71b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:rel:81b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:snp:91b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
     ],
 )
 def test_deposit_metadata_swhid(
-    swhid,
-    target_type,
-    authenticated_client,
-    deposit_collection,
-    atom_dataset,
-    swh_storage,
+    swhid, authenticated_client, deposit_collection, atom_dataset, swh_storage,
 ):
     """Posting a swhid reference is stored on raw extrinsic metadata storage
 
     """
-    swhid_reference = parse_swhid(swhid)
-    swhid_core = attr.evolve(swhid_reference, metadata={})
+    swhid_reference = QualifiedSWHID.from_string(swhid)
+    swhid_target = extended_swhid_from_qualified(swhid_reference)
 
     xml_data = atom_dataset["entry-data-with-swhid"].format(swhid=swhid)
     deposit_client = authenticated_client.deposit_client
 
+    _insert_object(swh_storage, swhid_reference)
+
     response = post_atom(
         authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
         data=xml_data,
     )
 
-    assert response.status_code == status.HTTP_201_CREATED
+    assert response.status_code == status.HTTP_201_CREATED, response.content.decode()
     response_content = parse_xml(BytesIO(response.content))
 
     # Ensure the deposit is finalized
     deposit_id = int(response_content["swh:deposit_id"])
     deposit = Deposit.objects.get(pk=deposit_id)
-    assert isinstance(swhid_core, SWHID)
-    assert deposit.swhid == str(swhid_core)
+    assert deposit.swhid == str(swhid_target)
     assert deposit.swhid_context == str(swhid_reference)
     assert deposit.complete_date == deposit.reception_date
     assert deposit.complete_date is not None
@@ -556,31 +592,31 @@ def test_deposit_metadata_swhid(
     )
     assert actual_fetcher == metadata_fetcher
 
+    # Get the deposited metadata object and check it:
+
     page_results = swh_storage.raw_extrinsic_metadata_get(
-        target_type, swhid_core, metadata_authority
+        swhid_target, metadata_authority
     )
-    discovery_date = page_results.results[0].discovery_date
 
     assert len(page_results.results) == 1
     assert page_results.next_page_token is None
 
-    object_type, metadata_context = compute_metadata_context(swhid_reference)
-    assert page_results == PagedResult(
-        results=[
-            RawExtrinsicMetadata(
-                type=object_type,
-                target=swhid_core,
-                discovery_date=discovery_date,
-                authority=attr.evolve(metadata_authority, metadata=None),
-                fetcher=attr.evolve(metadata_fetcher, metadata=None),
-                format="sword-v2-atom-codemeta",
-                metadata=xml_data.encode(),
-                **metadata_context,
-            )
-        ],
-        next_page_token=None,
+    metadata_context = compute_metadata_context(swhid_reference)
+    metadata = RawExtrinsicMetadata(
+        target=swhid_target,
+        discovery_date=deposit.complete_date,
+        authority=attr.evolve(metadata_authority, metadata=None),
+        fetcher=attr.evolve(metadata_fetcher, metadata=None),
+        format="sword-v2-atom-codemeta",
+        metadata=xml_data.encode(),
+        **metadata_context,
+    )
+    assert page_results == PagedResult(results=[metadata], next_page_token=None,)
+
+    # Get metadata about the deposited metadata object and check it:
+    _assert_deposit_info_on_metadata(
+        swh_storage, metadata.swhid(), deposit, metadata_fetcher
     )
-    assert deposit.complete_date == discovery_date
 
 
 @pytest.mark.parametrize(
@@ -593,14 +629,16 @@ def test_deposit_metadata_origin(
 
     """
     xml_data = atom_dataset["entry-data-with-origin-reference"].format(url=url)
+    origin_swhid = Origin(url).swhid()
     deposit_client = authenticated_client.deposit_client
+    swh_storage.origin_add([Origin(url)])
     response = post_atom(
         authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
         data=xml_data,
     )
 
-    assert response.status_code == status.HTTP_201_CREATED
+    assert response.status_code == status.HTTP_201_CREATED, response.content.decode()
     response_content = parse_xml(BytesIO(response.content))
     # Ensure the deposit is finalized
     deposit_id = int(response_content["swh:deposit_id"])
@@ -636,26 +674,112 @@ def test_deposit_metadata_origin(
     )
     assert actual_fetcher == metadata_fetcher
 
+    # Get the deposited metadata object and check it:
+
     page_results = swh_storage.raw_extrinsic_metadata_get(
-        MetadataTargetType.ORIGIN, url, metadata_authority
+        origin_swhid, metadata_authority
     )
-    discovery_date = page_results.results[0].discovery_date
 
     assert len(page_results.results) == 1
     assert page_results.next_page_token is None
 
-    assert page_results == PagedResult(
-        results=[
-            RawExtrinsicMetadata(
-                type=MetadataTargetType.ORIGIN,
-                target=url,
-                discovery_date=discovery_date,
-                authority=attr.evolve(metadata_authority, metadata=None),
-                fetcher=attr.evolve(metadata_fetcher, metadata=None),
-                format="sword-v2-atom-codemeta",
-                metadata=xml_data.encode(),
-            )
-        ],
-        next_page_token=None,
+    metadata = RawExtrinsicMetadata(
+        target=origin_swhid,
+        discovery_date=deposit.complete_date,
+        authority=attr.evolve(metadata_authority, metadata=None),
+        fetcher=attr.evolve(metadata_fetcher, metadata=None),
+        format="sword-v2-atom-codemeta",
+        metadata=xml_data.encode(),
     )
-    assert deposit.complete_date == discovery_date
+    assert page_results == PagedResult(results=[metadata], next_page_token=None,)
+
+    # Get metadata about the deposited metadata object and check it:
+    _assert_deposit_info_on_metadata(
+        swh_storage, metadata.swhid(), deposit, metadata_fetcher
+    )
+
+
+@pytest.mark.parametrize(
+    "swhid",
+    [
+        "swh:1:cnt:01b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:dir:11b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:rev:21b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:rel:31b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:snp:41b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:cnt:51b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:dir:c4993c872593e960dc84e4430dbbfbc34fd706d0;origin=https://inria.halpreprod.archives-ouvertes.fr/hal-01243573;visit=swh:1:snp:0175049fc45055a3824a1675ac06e3711619a55a;anchor=swh:1:rev:b5f505b005435fa5c4fa4c279792bd7b17167c04;path=/",  # noqa
+        "swh:1:rev:71b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:rel:81b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+        "swh:1:snp:91b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+    ],
+)
+def test_deposit_metadata_unknown_swhid(
+    swhid, authenticated_client, deposit_collection, atom_dataset, swh_storage,
+):
+    """Posting a swhid reference is rejected if the referenced object is unknown
+
+    """
+    xml_data = atom_dataset["entry-data-with-swhid"].format(swhid=swhid)
+
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=xml_data,
+    )
+
+    assert (
+        response.status_code == status.HTTP_400_BAD_REQUEST
+    ), response.content.decode()
+    response_content = parse_xml(BytesIO(response.content))
+    assert "object does not exist" in response_content["sword:error"]["atom:summary"]
+
+
+@pytest.mark.parametrize(
+    "swhid",
+    [
+        "swh:1:ori:01b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+        "swh:1:emd:11b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+    ],
+)
+def test_deposit_metadata_extended_swhid(
+    swhid, authenticated_client, deposit_collection, atom_dataset, swh_storage,
+):
+    """Posting a swhid reference is rejected if the referenced SWHID is
+    for an extended object type
+
+    """
+    xml_data = atom_dataset["entry-data-with-swhid"].format(swhid=swhid)
+
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=xml_data,
+    )
+
+    assert (
+        response.status_code == status.HTTP_400_BAD_REQUEST
+    ), response.content.decode()
+    response_content = parse_xml(BytesIO(response.content))
+    assert "Invalid SWHID reference" in response_content["sword:error"]["atom:summary"]
+
+
+def test_deposit_metadata_unknown_origin(
+    authenticated_client, deposit_collection, atom_dataset, swh_storage,
+):
+    """Posting a swhid reference is stored on raw extrinsic metadata storage
+
+    """
+    url = "https://gitlab.org/user/repo"
+    xml_data = atom_dataset["entry-data-with-origin-reference"].format(url=url)
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=xml_data,
+    )
+
+    assert (
+        response.status_code == status.HTTP_400_BAD_REQUEST
+    ), response.content.decode()
+    response_content = parse_xml(BytesIO(response.content))
+    assert "known to the archive" in response_content["sword:error"]["atom:summary"]
